// ------------------------------------------------------------------ //
/*
 	gcc -g -Wall -o test test.c -L /usr/X11R6/lib/ -lX11 -lXtst -lXmu 
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xos.h>
#include <X11/extensions/shape.h>
#include <X11/Xmu/WinUtil.h>
#include <stdio.h>
#include <stdlib.h>
#include <X11/cursorfont.h>

int screen = 0;

Window Select_Window(Display *dpy) {
  int status;
  Cursor cursor;
  XEvent event;
  Window target_win = None, root = RootWindow(dpy,screen);
  int buttons = 0;

  /* Make the target cursor */
  cursor = XCreateFontCursor(dpy, XC_crosshair);

  /* Grab the pointer using target cursor, letting it room all over */
  status = XGrabPointer(dpy, root, False,
			ButtonPressMask|ButtonReleaseMask, GrabModeSync,
			GrabModeAsync, root, cursor, CurrentTime);
  if (status != GrabSuccess) {
    fprintf(stderr, "error");
    return 0;
  }

  /* Let the user select a window... */
  while ((target_win == None) || (buttons != 0)) {
    /* allow one more event */
    XAllowEvents(dpy, SyncPointer, CurrentTime);
    XWindowEvent(dpy, root, ButtonPressMask|ButtonReleaseMask, &event);
    switch (event.type) {
    case ButtonPress:
      if (target_win == None) {
	target_win = event.xbutton.subwindow; /* window selected */
	if (target_win == None) target_win = root;
      }
      buttons++;
      break;
    case ButtonRelease:
      if (buttons > 0) /* there may have been some down before we started */
	buttons--;
       break;
    }
  } 

  XUngrabPointer(dpy, CurrentTime);      /* Done with pointer */

  return(target_win);
}



int main () {
  Display *dpy;
  Window win;
  char *data = "hello world\n";
  int i;

  dpy = XOpenDisplay (":0");

  win = Select_Window(dpy);
  win = XmuClientWindow (dpy, win);
  
  char* winname;
  if (XFetchName(dpy, win, &winname)) {
    printf("selected window: %s (%x)\n", winname, (uint)win);
  } else {
    printf("selected window: (no name) (%x)\n", (uint)win);
  }
  

  for (i = 0; i < strlen (data); i++)
    {
      XKeyEvent ev;
      KeySym key;

      ev.window = win;
      ev.root = DefaultRootWindow (dpy);
      ev.subwindow = win;
      ev.time = CurrentTime;
      ev.x = ev.y = ev.x_root = ev.y_root = 1;
      ev.state = 0;
      ev.same_screen = True;

      char str[512];
      sprintf (str, "%c", data[i]);
      key = XStringToKeysym (str);
      if (str[0] == ' ') {
        key = XK_space;
      }

// ignores, for example, XK_space, XK_comma...
      ev.keycode = XKeysymToKeycode (dpy, key);
      if (!ev.keycode)
	fprintf (stderr, "cannot find matching keycode");

      ev.type = KeyPress;
      XSendEvent (dpy, win, True, KeyPressMask, (XEvent *) & ev);
      ev.type = KeyRelease;
      XSendEvent (dpy, win, True, KeyReleaseMask, (XEvent *) & ev);

    }


  XCloseDisplay (dpy);

  return 0;
}


